using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using Barbar.MapReport.Common;

namespace Barbar.MapReport.Common.Drawing {
  /// <summary>
  /// Bod s business hodnotou
  /// </summary>
  [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1036:OverrideMethodsOnComparableTypes"), XmlType(Namespace = Constants.XmlNamespace)]
  public sealed class BusinessGeoPoint : IGeoPoint, IBusinessValue, IComparable<BusinessGeoPoint> {
    private decimal[] m_Values = new decimal[2];
    private decimal m_Longitude;
    private decimal m_Latitude;
    private string m_Name;
    private string m_ToolTip;
    private string m_ToolTipForAggregation;
    private string m_Hyperlink;

    /// <summary>
    /// Vchoz konstruktor
    /// </summary>
    public BusinessGeoPoint() {
    }

    /// <summary>
    /// Vytvor nov GeoValuePoint
    /// </summary>
    /// <param name="latitude">Zemepisn rka (sever-jih)</param>
    /// <param name="longitude">Zemepisn dlka (vchod-zpad)</param>
    /// <param name="value">Business hodnota</param>
    /// <param name="name">Nzev</param>
    /// <param name="description">Popis</param>
    /// <param name="hyperlink">Hyperlink, na kter bod odkazuje</param>
    public BusinessGeoPoint(decimal latitude, decimal longitude, decimal value, string name, string description, string hyperlink) {
      m_Latitude = latitude;
      m_Longitude = longitude;
      m_Values[0] = value;
      m_Name = name;
      m_ToolTip = description;
      m_Hyperlink = hyperlink;
    }

    #region IGeoPoint Members

    /// <summary>
    /// Zemepisn dlka (vchod-zpad)
    /// </summary>
    public decimal Longitude {
      get { return m_Longitude; }
      set { m_Longitude = value; }
    }

    /// <summary>
    /// Zemepisn rka (sever-jih)
    /// </summary>
    public decimal Latitude {
      get { return m_Latitude; }
      set { m_Latitude = value; }
    }

    #endregion

    /// <summary>
    /// Business hodnota
    /// </summary>
    public decimal Value1 {
      get { return m_Values[0]; }
      set { m_Values[0] = value; }
    }

    /// <summary>
    /// Business hodnota - pro dvojhodnotov indiktory
    /// </summary>
    public decimal Value2 {
      get { return m_Values[1]; }
      set { m_Values[1] = value; }
    }

    #region IValuable Members

    /// <summary>
    /// Nzev bodu
    /// </summary>
    public string Name {
      get { return m_Name; }
      set { m_Name = value; }
    }

    /// <summary>
    /// Popisek bodu
    /// </summary>
    public string ToolTip {
      get { return m_ToolTip; }
      set { m_ToolTip = value; }
    }

    /// <summary>
    /// Tooltip kter se zobraz, pokud se bude vce indiktoru hodnot prekrvat
    /// </summary>
    public string ToolTipForAggregation {
      get { return m_ToolTipForAggregation; }
      set { m_ToolTipForAggregation = value; }
    }

    /// <summary>
    /// Hyperlink, na kter se bod odkazuje
    /// </summary>
    public string Hyperlink {
      get { return m_Hyperlink; }
      set { m_Hyperlink = value; }
    }

    #endregion

    #region IComparable<GeoValuePoint> Members

    /// <summary>
    /// Porovn dva body dle jejich business hodnoty
    /// </summary>
    /// <param name="other">Druh bod</param>
    /// <returns>Vsledek porovnn</returns>
    public int CompareTo(BusinessGeoPoint other) {
      return m_Values[0].CompareTo(other.m_Values[0]);
    }

    #endregion

    #region IValuable Members

    decimal IBusinessValue.Value {
      get { return m_Values[0]; }
      set { m_Values[0] = value; }
    }

    #endregion
  }
}
