using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Drawing;

using System.Globalization;
using Barbar.MapReport.Common;

namespace Barbar.MapReport.Common.Drawing.Commands {
  /// <summary>
  /// Definuje geocru
  /// </summary>
  [XmlType(Namespace = Constants.XmlNamespace)]
  public class GeoLine {
    private GeoPoint m_StartPoint;
    private GeoPoint m_Endpoint;
    private Color m_Color;
    private int m_Offset;
    private int m_Width = 1;

    /// <summary>
    /// Vchoz konstruktor - jen pro xml serializaci
    /// </summary>
    public GeoLine() {
    }

    /// <summary>
    /// Konstruktor
    /// </summary>
    /// <param name="startPoint"></param>
    /// <param name="endpoint"></param>
    public GeoLine(IGeoPoint startPoint, IGeoPoint endpoint) {
      m_StartPoint = new GeoPoint(startPoint);
      m_Endpoint = new GeoPoint(endpoint);
    }

    /// <summary>
    /// Vchoz bod
    /// </summary>
    public GeoPoint StartPoint {
      get { return m_StartPoint; }
      set { m_StartPoint = value; }
    }

    /// <summary>
    /// Koncov bod
    /// </summary>
    public GeoPoint Endpoint {
      get { return m_Endpoint; }
      set { m_Endpoint = value; }
    }

    /// <summary>
    /// rka v pixelech
    /// </summary>
    public int Width {
      get { return m_Width; }
      set {
        if (m_Width < 1)
          throw new ArgumentOutOfRangeException("value");
        m_Width = value;
      }
    }

    /// <summary>
    /// Barva
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Usage", "CA1801:ReviewUnusedParameters", MessageId = "value")]
    public SerializedColor SerializedColor {
      get { return new SerializedColor(m_Color); }
      set { m_Color = value.Color; }
    }

    internal Color Color {
      get { return m_Color; }
    }

    /// <summary>
    /// posunut ipky oproti zactku cry v pixelech
    /// </summary>
    public int Offset {
      get { return m_Offset; }
      set { m_Offset = value; }
    }
  }
}
