using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Xml.Serialization;

namespace Barbar.MapReport.Common.Drawing {
  /// <summary>
  /// Popisek legendy
  /// </summary>
  [XmlType(Namespace = Constants.XmlNamespace)]
  [Serializable]
  public class LegendDescription : ILegendDescription {
    private Color m_Color;
    private string m_Title;
    private string m_UnitName;

    /// <summary>
    /// Vchoz konstruktor
    /// </summary>
    public LegendDescription() {
    }

    /// <summary>
    /// Konstruktor
    /// </summary>
    /// <param name="color"></param>
    /// <param name="title"></param>
    /// <param name="unitName"></param>
    public LegendDescription(Color color, string title, string unitName) {
      m_Color = color;
      m_Title = title;
      m_UnitName = unitName;
    }

    /// <summary>
    /// Konstruktor
    /// </summary>
    /// <param name="description"></param>
    public LegendDescription(ILegendDescription description) {
      if (description == null)
        throw new ArgumentNullException("description");
      m_Color = description.Color;
      m_Title = description.Title;
      m_UnitName = description.UnitName;
    }

    /// <summary>
    /// Barva
    /// </summary>
    public int Color {
      get { return m_Color.ToArgb(); }
      set { m_Color = System.Drawing.Color.FromArgb(value); }
    }

    #region ILegendDescription Members
    [XmlIgnore()]
    Color ILegendDescription.Color {
      get { return m_Color; }
      set { m_Color = value; }
    }

    /// <summary>
    /// Popisek
    /// </summary>
    public string Title {
      get { return m_Title; }
      set { m_Title = value; }
    }

    /// <summary>
    /// Nzev jednotky
    /// </summary>
    public string UnitName {
      get { return m_UnitName; }
      set { m_UnitName = value; }
    }

    #endregion
  }
}
