using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.Serialization;
using System.Drawing;
using System.Globalization;

namespace Barbar.MapReport.Common {
  /// <summary>
  /// Zkladn implementace geografickho bodu
  /// </summary>
  [XmlTypeAttribute(Namespace = Constants.XmlNamespace)]
  [Serializable]
  public class GeoPoint : IGeoPoint, IGeoBound {
    private decimal m_Longitude;
    private decimal m_Latitude;

    /// <summary>
    /// Vytvor nov geografick bod
    /// </summary>
    public GeoPoint() {
    }

    /// <summary>
    /// Vytvor nov geografick bod
    /// </summary>
    /// <param name="latitude">Zemepisn rka (sever-jih)</param>
    /// <param name="longitude">Zemepisn dlka (zpad-vchod)</param>
    public GeoPoint(decimal latitude, decimal longitude) {
      m_Latitude = latitude;
      m_Longitude = longitude;
    }

    /// <summary>
    /// Vytvor nov geografick bod (copy constructor)
    /// </summary>
    /// <param name="point">Geografick bod</param>
    public GeoPoint(IGeoPoint point) {
      m_Latitude = point.Latitude;
      m_Longitude = point.Longitude;
    }

    #region IGeoPoint Members

    /// <summary>
    /// Zemepisn dlka (vchod-zpad)
    /// </summary>
    public decimal Longitude {
      get { return m_Longitude; }
      set { m_Longitude = value; }
    }

    /// <summary>
    /// Zemepisn rka (sever-jih)
    /// </summary>
    public decimal Latitude {
      get { return m_Latitude; }
      set { m_Latitude = value; }
    }

    #endregion

    /// <summary>
    /// Porovn dva body (stejn jsou pokud maj stejnou Latitude, Longitude)
    /// </summary>
    /// <param name="obj"></param>
    /// <returns></returns>
    public override bool Equals(object obj) {
      IGeoPoint point = obj as IGeoPoint;
      if ((object)point == null)
        return false;
      return (point.Latitude == Latitude && point.Longitude == Longitude);
    }

    public override string ToString() {
      return String.Format(CultureInfo.InvariantCulture, "[lng={0};lat={1}]", m_Longitude, m_Latitude);
    }

    /// <summary>
    /// Vytvor hash kd (zaloen na kombinace Latitude, Longitude)
    /// </summary>
    /// <returns></returns>
    public override int GetHashCode() {
      return ((int)Latitude) ^ ((int)Longitude);
    }

    #region IGeoBound Members

    IGeoPoint IGeoBound.Northeast {
      get { return this; }
    }

    IGeoPoint IGeoBound.Southwest {
      get { return this; }
    }

    #endregion
  }
}
