using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;
using System.Globalization;

namespace Barbar.MapReport.Common {
  /// <summary>
  /// Struktura reprezentujc bod v rovine mapy
  /// </summary>
  [Serializable]
  public struct PointD {
    /// <summary>
    /// X-ov souradnice bodu
    /// </summary>
    public double X;
    /// <summary>
    /// Y-ov souradnice bodu
    /// </summary>
    public double Y;

    /// <summary>
    /// Konstruktor
    /// </summary>
    /// <param name="x">x-ov souradnice bodu</param>
    /// <param name="y">y-ov souradnice bodu</param>
    public PointD(double x, double y) {
      X = x;
      Y = y;
    }

    public PointD(VectorD vector) {
      X = vector.X;
      Y = vector.Y;
    }

    public override string ToString() {
      return String.Format(CultureInfo.InvariantCulture, "[{0};{1}]", X, Y);
    }

    public override int GetHashCode() {
      return X.GetHashCode() ^ Y.GetHashCode();
    }

    /// <summary>
    /// Secte dva body v rovinne
    /// </summary>
    /// <param name="point">Bod k pricten</param>
    /// <returns>Bod jeho souradnice vzniknou souctem</returns>
    public PointD Add(PointD point) {
      return new PointD(X + point.X, Y + point.Y);
    }

    /// <summary>
    /// Vydel bod skalrn hodnotou
    /// </summary>
    /// <param name="value">Delc hodnota</param>
    /// <returns>Bod jeho souradnice vzniknou delenm</returns>
    public PointD Divide(double value) {
      return new PointD(X / value, Y / value);
    }

    /// <summary>
    /// Odecte dva body v rovinne
    /// </summary>
    /// <param name="point">Bod k odecten</param>
    /// <returns>Bod jeho souradnice vzniknou rozdlem</returns>
    public VectorD Substract(PointD point) {
      return new VectorD(X - point.X, Y - point.Y);
    }

    /// <summary>
    /// Zkonvertuje to celocseln struktury
    /// </summary>
    /// <returns>Bod reprezentovan celocselnou strukturou</returns>
    public Point ToPoint() {
      return new Point((int)X, (int)Y);
    }

    public static PointD operator+(PointD point, VectorD vector) {
      return new PointD(point.X + vector.X, point.Y + vector.Y);
    }

    public static VectorD operator -(PointD start, PointD end) {
      return new VectorD(end.X - start.X, end.Y - start.Y);
    }
  }
}
