using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

namespace Barbar.MapReport.DataSource.Esri {
  /// <summary>
  /// Hlavicka Shp zznamu
  /// </summary>
  public class ShpRecordHeader {
    [ShpType(ShpByteOrder.BigEndian, ShpAtomicType.Integer)]
    private int m_RecordNumber;
    [ShpType(ShpByteOrder.BigEndian, ShpAtomicType.Integer)]
    private int m_ContentLength;

    public int RecordNumber {
      get { return m_RecordNumber; }
      set { m_RecordNumber = value; }
    }

    public int ContentLength {
      get { return m_ContentLength; }
      set { m_ContentLength = value; }
    }

    internal ShpRecordHeader(Stream stream) {
      FieldInfo[] fields = GetType().GetFields(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
      foreach (FieldInfo fieldInfo in fields) {
        ShpTypeAttribute[] shpInfos = (ShpTypeAttribute[])fieldInfo.GetCustomAttributes(typeof(ShpTypeAttribute), false);
        if (shpInfos != null && shpInfos.Length > 0) {
          if (shpInfos.Length > 1)
            throw new ShpException("ShpRecord declared more than once for one attribute.");
          ShpTypeAttribute shpInfo = shpInfos[0];
          object value = shpInfo.LoadType(stream);
          fieldInfo.SetValue(this, value);
        }
      }
    }
  }
}
