using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Reflection;

namespace Barbar.MapReport.DataSource.Esri {
  /// <summary>
  /// Reprezentuje zznam typu polygon
  /// </summary>
  internal class ShpRecordPolygon {
    [ShpType(ShpByteOrder.LittleEndian, ShpAtomicType.Double)]
    private double m_Xmin;
    [ShpType(ShpByteOrder.LittleEndian, ShpAtomicType.Double)]
    private double m_Ymin;
    [ShpType(ShpByteOrder.LittleEndian, ShpAtomicType.Double)]
    private double m_Xmax;
    [ShpType(ShpByteOrder.LittleEndian, ShpAtomicType.Double)]
    private double m_Ymax;
    [ShpType(ShpByteOrder.LittleEndian, ShpAtomicType.Integer)]
    private int m_NumParts;
    [ShpType(ShpByteOrder.LittleEndian, ShpAtomicType.Integer)]
    private int m_NumPoints;
    private int[] m_Parts;
    private ShpPoint[] m_Points;
    private ShpPolygon m_Polygon;

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
    public double Xmin {
      get { return m_Xmin; }
      set { m_Xmin = value; }
    }

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
    public double Ymin {
      get { return m_Ymin; }
      set { m_Ymin = value; }
    }

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
    public double Xmax {
      get { return m_Xmax; }
      set { m_Xmax = value; }
    }

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
    public double Ymax {
      get { return m_Ymax; }
      set { m_Ymax = value; }
    }

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
    public int NumParts {
      get { return m_NumParts; }
      set { m_NumParts = value; }
    }

    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode"), System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Performance", "CA1811:AvoidUncalledPrivateCode")]
    public int NumPoints {
      get { return m_NumPoints; }
      set { m_NumPoints = value; }
    }

    /// <summary>
    /// Vrt objekt, reprezentujc polygon
    /// </summary>
    public ShpPolygon Polygon {
      get {
        if (m_Polygon == null) {
          m_Polygon = new ShpPolygon(m_Parts, m_Points);
        }
        return m_Polygon;
      }
    }

    internal ShpRecordPolygon(Stream stream) {
      FieldInfo[] fields = GetType().GetFields(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
      foreach (FieldInfo fieldInfo in fields) {
        ShpTypeAttribute[] shpInfos = (ShpTypeAttribute[])fieldInfo.GetCustomAttributes(typeof(ShpTypeAttribute), false);
        if (shpInfos != null && shpInfos.Length > 0) {
          if (shpInfos.Length > 1)
            throw new ShpException("ShpRecord declared more than once for one attribute.");
          ShpTypeAttribute shpInfo = shpInfos[0];
          object value = shpInfo.LoadType(stream);
          fieldInfo.SetValue(this, value);
        }
      }

      m_Parts = new int[m_NumParts];
      for (int i = 0; i < m_NumParts; i++) {
        m_Parts[i] = (int)ShpUtility.LoadType(stream, ShpByteOrder.LittleEndian, ShpAtomicType.Integer);
      }
      
      m_Points = new ShpPoint[m_NumPoints];
      for (int i = 0; i < m_NumPoints; i++) {
        m_Points[i].Y = (double)ShpUtility.LoadType(stream, ShpByteOrder.LittleEndian, ShpAtomicType.Double);
        m_Points[i].X = (double)ShpUtility.LoadType(stream, ShpByteOrder.LittleEndian, ShpAtomicType.Double);
      }
    }
  }
}
