using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using Barbar.MapReport.Common;
using Barbar.MapReport.HtmlRender.Handlers;
using Barbar.MapReport.HtmlRender.Shapes;

namespace Barbar.MapReport.HtmlRender {
  /// <summary>
  /// HttpHandler pro podpurn cinnosti MAP API (poskytuje cache, java scripty, obrzky apod.)
  /// </summary>
  public sealed class Handler : IHttpHandler {
    private ISimpleHttpHandler[] m_PrivateHandlers = new ISimpleHttpHandler[] {
      new FileHandler(),
      new ShapeHandler(),
      new PageHandler(),
      new MapCallbackHandler()
    };

    #region IHttpHandler Members

    bool IHttpHandler.IsReusable {
      get { return false; }
    }

    void IHttpHandler.ProcessRequest(HttpContext context) {
      string type = context.Request["type"];
      if (String.IsNullOrEmpty(type))
        context.Response.Write("No type specified.");

      foreach (ISimpleHttpHandler handler in m_PrivateHandlers) {
        if (handler.GetType().FullName == type) {
          handler.ProcessRequest(context);
          return;
        }
      }
      throw new MapReportException("Type not known.");
    }

    #endregion
  }
}
