using System;
using System.Collections.Generic;
using System.Text;
using System.Web;
using System.Xml.Serialization;
using System.Globalization;
using System.ComponentModel;
using Barbar.MapReport.HtmlRender.Handlers;
using Barbar.MapReport.Common;

namespace Barbar.MapReport.HtmlRender {
  /// <summary>
  /// Formtovn vstupnho html
  /// </summary>
  [XmlTypeAttribute(Namespace = "http://codeplex.com/MapReport/Service.asmx")]
  public class RenderSettings {
    private SerializedUnit m_MapWidth = new SerializedUnit(800);
    private SerializedUnit m_MapHeight = new SerializedUnit(600);
    private decimal m_CentralLatitude = 50m;
    private decimal m_CentralLongitude = 14m;
    private int m_Zoom = 7;
    private string m_GoogleApiKey;
    private int m_LocalCultureId = 1033;
    private string m_MapProxyUrl;
    private bool m_ShowLegend = true;
    private string m_StyleText;
    private bool m_EnableGoogleBar;
    private MapControlType m_MapControlType = MapControlType.Small;
    private string m_MapObjectName = "map";
    private bool m_ShowSearchBox;

    /// <summary>
    /// Vchoz konstruktor
    /// </summary>
    public RenderSettings() {
      if (HttpContext.Current != null)
        m_MapProxyUrl = HandlerUrl.Url;
    }

    /// <summary>
    /// Konstruktor
    /// </summary>
    /// <param name="width"></param>
    /// <param name="height"></param>
    /// <param name="centerLatitude"></param>
    /// <param name="centerLongitude"></param>
    /// <param name="zoom"></param>
    /// <param name="googleApiKey"></param>
    /// <param name="localCultureId"></param>
    public RenderSettings(SerializedUnit width, SerializedUnit height, decimal centerLatitude, decimal centerLongitude, int zoom, string googleApiKey, int localCultureId)
      : this() {
      this.MapWidth = width;
      this.MapHeight = height;
      this.CentralLatitude = centerLatitude;
      this.CentralLongitude = centerLongitude;
      this.Zoom = zoom;
      this.GoogleApiKey = googleApiKey;
      this.LocalCultureId = localCultureId;
    }

    /// <summary>
    /// rka mapy
    /// </summary>
    public SerializedUnit MapWidth {
      get { return m_MapWidth; }
      set { m_MapWidth = value; }
    }

    /// <summary>
    /// Vka mapy
    /// </summary>
    public SerializedUnit MapHeight {
      get { return m_MapHeight; }
      set { m_MapHeight = value; }
    }

    /// <summary>
    /// Centrln bod mapy - zemepisn rka
    /// </summary>
    public decimal CentralLatitude {
      get { return m_CentralLatitude; }
      set { m_CentralLatitude = value; }
    }

    /// <summary>
    /// Centrln bod mapy - zemepisn vka
    /// </summary>
    public decimal CentralLongitude {
      get { return m_CentralLongitude; }
      set { m_CentralLongitude = value; }
    }

    /// <summary>
    /// Zoom mapy
    /// </summary>
    public int Zoom {
      get { return m_Zoom; }
      set { m_Zoom = value; }
    }

    /// <summary>
    /// Google klc
    /// </summary>
    public string GoogleApiKey {
      get { return m_GoogleApiKey; }
      set { m_GoogleApiKey = value; }
    }

    /// <summary>
    /// Kd kultury (jazykov lokalizace)
    /// </summary>
    public int LocalCultureId {
      get { return m_LocalCultureId; }
      set { m_LocalCultureId = value; }
    }

    /// <summary>
    /// Proxy kter preposl veker requesty mapreportu
    /// </summary>
    [System.Diagnostics.CodeAnalysis.SuppressMessage("Microsoft.Design", "CA1056:UriPropertiesShouldNotBeStrings")]
    public string MapProxyUrl {
      get { return m_MapProxyUrl; }
      set { m_MapProxyUrl = value; }
    }

    /// <summary>
    /// True pro zobrazen legendy
    /// </summary>
    public bool ShowLegend {
      get { return m_ShowLegend; }
      set { m_ShowLegend = value; }
    }

    /// <summary>
    /// Css styl pridan na zobrazovanou strnku
    /// </summary>
    public string StyleText {
      get { return m_StyleText; }
      set { m_StyleText = value; }
    }

    /// <summary>
    /// True pro zobrazen google baru na mape
    /// </summary>
    public bool EnableGoogleBar {
      get { return m_EnableGoogleBar; }
      set { m_EnableGoogleBar = value; }
    }

    public bool ShowSearchBox {
      get { return m_ShowSearchBox; }
      set { m_ShowSearchBox = value; }
    }

    /// <summary>
    /// Zapn nebo vypn zobrazen podrobn kontrolky na mape
    /// </summary>
    [DefaultValue(typeof(MapControlType), "Small")]
    public MapControlType MapControlType {
      get { return m_MapControlType; }
      set { m_MapControlType = value; }
    }

    public string MapObjectName {
      get { return m_MapObjectName; }
      set { m_MapObjectName = value; }
    }

    internal ReplacementTemplate CreateTemplate() {
      ReplacementTemplate result = new ReplacementTemplate();

      result.AddMessage("MapWidth", m_MapWidth.ToString());
      result.AddMessage("MapHeight", m_MapHeight.ToString());
      result.AddMessage("CentralLatitude", Convert.ToString(m_CentralLatitude, CultureInfo.InvariantCulture));
      result.AddMessage("CentralLongitude", Convert.ToString(m_CentralLongitude, CultureInfo.InvariantCulture));
      result.AddMessage("Zoom", Convert.ToString(m_Zoom, CultureInfo.InvariantCulture));
      result.AddMessage("GoogleApiKey", m_GoogleApiKey);

      CultureInfo cultureInfo = new CultureInfo(m_LocalCultureId);
      result.AddMessage("LocalCulture", cultureInfo.TwoLetterISOLanguageName);

      result.AddMessage("MapProxyUrl", m_MapProxyUrl);
      result.AddMessage("StyleText", m_StyleText);
      result.AddMessage("MapObjectName", m_MapObjectName);

      result.AddCondition("ShowLegend", m_ShowLegend);
      result.AddCondition("ShowSearchBox", m_ShowSearchBox);
      result.AddCondition("EnableGoogleBar", m_EnableGoogleBar);
      result.AddCondition("IsSmallMapControlEnabled", m_MapControlType == MapControlType.Small);
      result.AddCondition("IsLargeMapControlEnabled", m_MapControlType == MapControlType.Large);

      return result;
    }
  }
}
